@@ -70,7 +70,6 @@
 
 #include <libedataserver/e-msgport.h>
 
-#include "mail-component.h"
 #include "mail/mail-config.h"
 #include "mail/mail-mt.h"
 
@@ -152,6 +151,8 @@ efh_init(GObject *o)
 	guint32 rgb;
 
 	efh->priv = g_malloc0(sizeof(*efh->priv));
+	efh->show_photo = FALSE; /* Default photo isn't shown */
+	efh->local_only_photo = TRUE; /* Default photo isn't shown */
 
 	e_dlist_init(&efh->pending_object_list);
 	e_dlist_init(&efh->priv->pending_jobs);
@@ -278,7 +279,6 @@ em_format_mail_get_type(void)
 		//gchar *path;
 
 		/* Trigger creation of mail component. */
-		mail_component_peek ();
 
 		efh_parent = g_type_class_ref(em_format_get_type());
 		type = g_type_register_static(em_format_get_type(), "EMFormatMail", &info, 0);
@@ -1071,20 +1071,28 @@ emfh_write_related(EMFormat *emf, CamelStream *stream, EMFormatPURI *puri)
 static void
 emfh_multipart_related_check(struct _EMFormatMailJob *job, gint cancelled)
 {
-	struct _EMFormatPURITree *ptree;
-	EMFormatPURI *puri, *purin;
 	gchar *oldpartid;
+	GList *link;
+	EMFormat *format;
 
 	if (cancelled)
 		return;
+	
+	format = EM_FORMAT (job->format);
 
 	d(printf(" running multipart/related check task\n"));
 	oldpartid = g_strdup(((EMFormat *)job->format)->part_id->str);
 
-	ptree = job->puri_level;
-	puri = (EMFormatPURI *)ptree->uri_list.head;
-	purin = puri->next;
-	while (purin) {
+	link = g_queue_peek_head_link (job->puri_level->data);
+	if (!link) {
+		g_string_printf (format->part_id, "%s", oldpartid);
+		g_free (oldpartid);
+		return;
+	}
+
+	while (link->next != NULL) {
+		EMFormatPURI *puri = link->data;
+
 		if (puri->use_count == 0) {
 			d(printf("part '%s' '%s' used '%d'\n", puri->uri?puri->uri:"", puri->cid, puri->use_count));
 			if (puri->func == emfh_write_related) {
@@ -1093,8 +1101,8 @@ emfh_multipart_related_check(struct _EMFormatMailJob *job, gint cancelled)
 			}
 			/* else it was probably added by a previous format this loop */
 		}
-		puri = purin;
-		purin = purin->next;
+
+		link = g_list_next (link);
 	}
 
 	g_string_printf(((EMFormat *)job->format)->part_id, "%s", oldpartid);
@@ -1199,6 +1207,7 @@ efh_image(EMFormatMail *efh, CamelStream *stream, CamelMimePart *part, EMFormatH
 }
 
 static EMFormatHandler type_builtin_table[] = {
+#if 0
 	{ (gchar *) "image/gif", (EMFormatFunc)efh_image },
 	{ (gchar *) "image/jpeg", (EMFormatFunc)efh_image },
 	{ (gchar *) "image/png", (EMFormatFunc)efh_image },
@@ -1214,6 +1223,7 @@ static EMFormatHandler type_builtin_table[] = {
 	{ (gchar *) "image/x-portable-graymap", (EMFormatFunc)efh_image },
 	{ (gchar *) "image/x-portable-pixmap", (EMFormatFunc)efh_image },
 	{ (gchar *) "image/x-xpixmap", (EMFormatFunc)efh_image },
+#endif	
 	{ (gchar *) "text/enriched", (EMFormatFunc)efh_text_enriched },
 	{ (gchar *) "text/plain", (EMFormatFunc)efh_text_plain },
 	{ (gchar *) "text/html", (EMFormatFunc)efh_text_html },
@@ -1226,10 +1236,10 @@ static EMFormatHandler type_builtin_table[] = {
 	/* This is where one adds those busted, non-registered types,
 	   that some idiot mailer writers out there decide to pull out
 	   of their proverbials at random. */
-
+#if 0
 	{ (gchar *) "image/jpg", (EMFormatFunc)efh_image },
 	{ (gchar *) "image/pjpeg", (EMFormatFunc)efh_image },
-
+#endif
 	/* special internal types */
 
 	{ (gchar *) "x-evolution/message/rfc822", (EMFormatFunc)efh_format_message }
@@ -1268,7 +1278,7 @@ static void
 efh_format_exec (struct _format_msg *m)
 {
 	struct _EMFormatMailJob *job;
-	struct _EMFormatPURITree *puri_level;
+	GNode *puri_level;
 	gint cancelled = FALSE;
 	CamelURL *base;
 
@@ -1437,10 +1447,11 @@ efh_format_timeout(struct _format_msg *m)
 		mail_msg_unref(m);
 		p->last_part = NULL;
 	} else {
+		GtkWidget *web = mail_message_view_create_webview ((MailMessageView *)efh->msg_view, efh->body);
 		efh->state = EM_FORMAT_MAIL_STATE_RENDERING;
-		m->estream = (CamelStream *)mail_message_view_create_webstream((gpointer)mail_message_view_create_webview ((MailMessageView *)efh->msg_view, efh->body), efh->body);
+		m->estream = (CamelStream *)mail_message_view_create_webstream((gpointer)web, efh->body);
 		efh->header_stream = mail_message_view_create_webstream((gpointer)mail_message_view_create_webview ((MailMessageView *)efh->msg_view, efh->wheaders), efh->wheaders);
-
+		gtk_widget_show(web);
 #if 0
 		if (p->last_part != m->message) {
 			hstream = gtk_html_begin (efh->html);
@@ -1874,7 +1885,6 @@ static void
 efh_format_headers(EMFormatMail *efh, CamelStream *stream, CamelMedium *part)
 {
 	EMFormat *emf = (EMFormat *) efh;
-	EMFormatHeader *h;
 	const gchar *charset;
 	CamelContentType *ct;
 	struct _camel_header_raw *header;
@@ -1964,7 +1974,6 @@ efh_format_headers(EMFormatMail *efh, CamelStream *stream, CamelMedium *part)
 		camel_stream_printf (stream, "<tr><td><table border=0 cellpadding=\"0\">\n");
 
 	/* dump selected headers */
-	h = (EMFormatHeader *)emf->header_list.head;
 	if (emf->mode == EM_FORMAT_ALLHEADERS) {
 		header = ((CamelMimePart *)part)->headers;
 		while (header) {
@@ -1972,8 +1981,12 @@ efh_format_headers(EMFormatMail *efh, CamelStream *stream, CamelMedium *part)
 			header = header->next;
 		}
 	} else {
+		GList *link;
 		gint mailer_shown = FALSE;
-		while (h->next) {
+
+		link = g_queue_peek_head_link (&emf->header_list);
+		while (link != NULL) {
+			EMFormatHeader *h = link->data;
 			gint mailer, face;
 
 			header = ((CamelMimePart *)part)->headers;
@@ -1981,7 +1994,7 @@ efh_format_headers(EMFormatMail *efh, CamelStream *stream, CamelMedium *part)
 			face = !g_ascii_strcasecmp (h->name, "Face");
 
 			while (header) {
-				if (emf->show_photo && !photo_name && !g_ascii_strcasecmp (header->name, "From"))
+				if (efh->show_photo&& !photo_name && !g_ascii_strcasecmp (header->name, "From"))
 					photo_name = header->value;
 
 				if (!mailer_shown && mailer && (!g_ascii_strcasecmp (header->name, "X-Mailer") ||
@@ -2029,7 +2042,7 @@ efh_format_headers(EMFormatMail *efh, CamelStream *stream, CamelMedium *part)
 
 				header = header->next;
 			}
-			h = h->next;
+			link = g_list_next (link);
 		}
 	}
 
@@ -2042,7 +2055,7 @@ efh_format_headers(EMFormatMail *efh, CamelStream *stream, CamelMedium *part)
 
 			cia = camel_internet_address_new();
 			camel_address_decode((CamelAddress *) cia, (const gchar *) photo_name);
-			photopart = em_utils_contact_photo (cia, emf->photo_local);
+			photopart = em_utils_contact_photo (cia, efh->local_only_photo);
 
 			if (photopart) {
 				contact_has_photo = TRUE;
@@ -2174,7 +2187,6 @@ static void efh_format_source(EMFormat *emf, CamelStream *stream, CamelMimePart
 static void
 efh_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const gchar *mime_type, const EMFormatHandler *handle)
 {
-	gchar *text, *html;
 #if 0
 	/* we display all inlined attachments only */
 
