@@ -17,6 +17,7 @@
  *		Srinivasa Ragavan <sragavan@novell.com>
  *
  * Copyright (C) 2009 Novell, Inc. (www.novell.com)
+ * Copyright (C) 2009 Intel Corporation (www.intel.com)
  *
  */
 
@@ -27,7 +28,6 @@
 #include "mail-view.h"
 #include "mail-folder-view.h"
 #include "mail/em-folder-tree.h"
-#include "mail/em-popup.h"
 #include "mail/mail-mt.h"
 #include "mail/mail-ops.h"
 #include "mail/em-utils.h"
@@ -39,7 +39,6 @@
 #include <e-util/e-marshal.h>
 #endif
 #include "misc/e-spinner.h"
-#include "mail-sqlite-store.h"
 #include "em-tree-store.h"
 #include "custom-cell-renderer-hbox.h"
 #include "custom-cell-renderer-vbox.h"
@@ -48,6 +47,7 @@
 #include "mail-utils.h"
 #include "mail-composer-view.h"
 #include "mail-shell.h"
+#include "mail-sqlite-store.h"
 
 #include <gdk/gdkkeysyms.h>
 
@@ -71,7 +71,7 @@ struct  _MailFolderViewPrivate {
 	GtkWidget *view_scroller;
 	GtkWidget *info_bar;
 	GtkWidget *info_spinner;
-	CamelOperation *cancel;
+	gint cancel;
 	GtkCellRenderer *from_to;
 	GtkTreeViewColumn *cols[3];
 
@@ -215,14 +215,23 @@ mfv_close (GtkButton *w, MailFolderView *mfv)
 }
 
 static char *
-mfv_folder_name (char *name)
+mfv_folder_name (CamelStore *store, char *name, char *uri)
 {
-	if (strcmp(name, CAMEL_VTRASH_NAME) == 0)
-		return _("Trash");
-	else if (strcmp (name, CAMEL_VJUNK_NAME) == 0)
-		return _("Junk");
+	char *new_name;
 
-	return name;
+	if (strcmp(name, CAMEL_VTRASH_NAME) == 0)
+		name = _("Trash");
+	else if (strcmp (name, CAMEL_VJUNK_NAME) == 0)
+		name = _("Junk");
+
+	if (uri && strncmp (uri, "mbox", 4) == 0) {
+		new_name = g_strdup_printf ("%s %s '%s'", name, _("on"), _("On This Computer"));
+	} else {
+		char *tmpstr = camel_service_get_name((CamelService *)store, TRUE);
+		new_name = g_strdup_printf ("%s %s '%s'", name, _("on"), tmpstr);
+		g_free(tmpstr);
+	}
+	return new_name;
 }
 
 static void
@@ -260,7 +269,7 @@ mail_folder_view_init (MailFolderView  *shell)
 	shell->priv->show_children = g_getenv("ANJAL_CHILDREN") != NULL;
 	shell->priv->folder = NULL;
 	shell->search = NULL;
-	shell->priv->cancel = NULL;
+	shell->priv->cancel = 0;
 	if (shell->priv->light) 
 		shell->model = mail_sqlite_store_new ();
 	else {
@@ -302,11 +311,16 @@ mail_folder_view_finalize (GObject *object)
 {
 	 MailFolderView *mfv = (MailFolderView  *)object;
 
+	 if (mfv->priv->unread_toggle)
+		 g_object_unref (mfv->priv->unread_toggle);
+	 if (mfv->priv->from_to)
+		 g_object_unref (mfv->priv->from_to);
+
 	 if (mfv->priv->folder_name)
 		 g_free (mfv->priv->folder_name);
 
 	 if (mfv->priv->cancel) {
-		camel_operation_cancel(mfv->priv->cancel);
+		mail_msg_cancel (mfv->priv->cancel);
 	 }
 	 if (mfv->search_state) {
 		 g_free(mfv->search_state);
@@ -457,6 +471,7 @@ mail_folder_view_set_folder_uri (MailFolderView *mfv, const char *uri)
 	if (uri && mfv->uri && !strcmp(uri, mfv->uri))
 		 return;
 
+	printf("%s %s\n", uri, mfv->uri);
 	if (mfv->uri)
 		 g_free(mfv->uri);
 	mfv->uri = g_strdup(uri);
@@ -468,6 +483,7 @@ mail_folder_view_set_folder_uri (MailFolderView *mfv, const char *uri)
 	if (uri) {
 		str = g_strdup_printf("%s %s", _("Loading folder"), uri);
 		gtk_label_set_text (g_object_get_data((GObject *)mfv->priv->info_bar, "label"), str);
+		gtk_label_set_text ((GtkLabel *)mfv->priv->tab_str, str);
 		g_free(str);
 		mail_get_folder(uri, 0, mfv_got_folder, mfv, mail_msg_fast_ordered_push);
 	}
@@ -479,12 +495,15 @@ mfv_folder_renamed (CamelStore *store, void *event, void *data)
 	MailFolderView *mfv = (MailFolderView *)data;
 	CamelRenameInfo *info = (CamelRenameInfo *)event;
 	if (g_strcmp0(info->old_base, mfv->priv->folder_name) == 0){
+		char *tmpstr;
 		/* Folder renamed. Revamp the uri.*/
 		g_free(mfv->uri);
 		g_free(mfv->priv->folder_name);
 		mfv->uri = g_strdup(info->new->uri);
 		mfv->priv->folder_name = g_strdup (info->new->full_name);
-		gtk_label_set_text ((GtkLabel *)mfv->priv->tab_str, mfv_folder_name(info->new->full_name));
+		tmpstr = mfv_folder_name(store, info->new->full_name, mfv->uri);
+		gtk_label_set_text ((GtkLabel *)mfv->priv->tab_str, tmpstr);
+		g_free(tmpstr);
 	}
 }
 
@@ -510,7 +529,8 @@ mail_folder_view_set_folder (MailFolderView *mfv, CamelFolder *folder, const cha
 		double pos = 0;
 		GtkTreeIter iter;
 		gboolean outgoing = FALSE;
-		
+		char *tmp_tabstr;
+
 	        if (em_utils_folder_is_drafts(folder, uri)
 			|| em_utils_folder_is_sent(folder, uri)
 			|| em_utils_folder_is_outbox(folder, uri))
@@ -539,7 +559,10 @@ mail_folder_view_set_folder (MailFolderView *mfv, CamelFolder *folder, const cha
 		if (spos && *spos) {
 			pos = strtod(spos, NULL);
 		}
-		gtk_label_set_text ((GtkLabel *)mfv->priv->tab_str, mfv_folder_name(folder->full_name));
+		tmp_tabstr = mfv_folder_name(folder->parent_store, folder->full_name, mfv->uri);
+		gtk_label_set_text ((GtkLabel *)mfv->priv->tab_str, tmp_tabstr);
+		g_free (tmp_tabstr);
+
 		gtk_adjustment_set_value(gtk_scrolled_window_get_vadjustment ((GtkScrolledWindow *)mfv->priv->view_scroller), pos);
 		if (last_sel_uid && *last_sel_uid) {
 			 GtkTreeSelection *sel = gtk_tree_view_get_selection (mfv->tree);
@@ -554,7 +577,7 @@ mail_folder_view_set_folder (MailFolderView *mfv, CamelFolder *folder, const cha
 			 }
 		}
 		gtk_widget_grab_focus ((GtkWidget *)mfv->tree);
-		if (mfv->search_str && *mfv->search_str)
+		//if (mfv->search_str && *mfv->search_str)
 			g_signal_emit (mfv, signals[SEARCH_SET], 0);			
 
 		/* FIXME: Set a signal emit and listen for it at note book and set the title appropriately */
@@ -699,8 +722,11 @@ draw_img (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell, GtkTreeModel *t
 		  if (data) {
 			  img = gtk_widget_render_icon (wid, data, GTK_ICON_SIZE_MENU, NULL);
 			  g_object_set ((GObject *)child, "pixbuf", img, NULL);
+			  g_object_unref (img);
 		  } else {
-			  g_object_set ((GObject *)child, "pixbuf", gdk_pixbuf_new_from_xpm_data ((const char **) empty_xpm), NULL);
+			  GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) empty_xpm);
+			  g_object_set ((GObject *)child, "pixbuf", pixbuf, NULL);
+			  g_object_unref (pixbuf);
 		  }
 		  tmp = tmp->next;
 	}
@@ -901,9 +927,13 @@ draw_text (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell, GtkTreeModel *
 						 
 						 gtk_tree_model_get_value (tree_model, iter, EMTS_COL_ATTACHMENT, &v1);
 						 if (g_value_get_boolean (&v1)) {
-			 				g_object_set ((GObject *)c, "pixbuf", mail_utils_get_icon("mail-attachment", GTK_ICON_SIZE_MENU), NULL);
+							GdkPixbuf *icon = mail_utils_get_icon("mail-attachment", GTK_ICON_SIZE_MENU);
+			 				g_object_set ((GObject *)c, "pixbuf", icon, NULL);
+							g_object_unref (icon);
 		  				 } else {
-			  				g_object_set ((GObject *)c, "pixbuf", gdk_pixbuf_new_from_xpm_data ((const char **) empty_xpm), NULL);
+							GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) empty_xpm);
+			  				g_object_set ((GObject *)c, "pixbuf", pixbuf, NULL);
+							g_object_unref (pixbuf);
 		  				 }
 					}
 					i=-1;
@@ -966,7 +996,8 @@ draw_flags (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell, GtkTreeModel
 void
 mail_folder_view_show_list (MailFolderView *mfv)
 {
-	 mfv->priv->message_shown = 0;
+	mfv->priv->message_shown = 0;
+	gtk_widget_hide (mfv->slidebox);
 	gtk_widget_show ((GtkWidget *)mfv->tree);
 	gtk_widget_show (mfv->priv->view_scroller);
 	gtk_widget_hide ((GtkWidget *)mfv->mail);
@@ -1024,6 +1055,27 @@ mfv_get_selected_threads (MailFolderView *mfv)
 	return uids;
 }
 
+static void
+mfv_message_got(CamelFolder *folder, const char *uid, CamelMimeMessage *msg, void *data, CamelException *ex)
+{
+	EMsgComposer *composer;
+	MailFolderView *mfv = (MailFolderView *)data;
+	MailComposerView *mcv;
+
+	mfv->priv->cancel = 0;
+	composer = (EMsgComposer *)em_utils_edit_message (msg, folder);
+	mcv = (MailComposerView *)mail_shell_create_composer ();
+	mail_composer_view_replace_composer ((struct _MailComposerView *)mcv, (GtkWidget *)composer);
+}
+
+static void
+mfv_open_composer (MailFolderView *mfv, char *uid)
+{
+	e_msg_composer_set_lite();
+	mfv->priv->cancel = mail_get_messagex (mfv->priv->folder, uid, mfv_message_got, mfv, mail_msg_fast_ordered_push);
+}
+
+#if 0
 static void
 mfv_popup_open_new_tab (EPopup *ep, EPopupItem *pitem, void *data)
 {
@@ -1106,25 +1158,7 @@ mfv_popup_undelete (EPopup *ep, EPopupItem *pitem, void *data)
 	 mfv_delete (mfv, FALSE);
 }
 
-static void
-mfv_message_got(CamelFolder *folder, const char *uid, CamelMimeMessage *msg, void *data, CamelException *ex)
-{
-	EMsgComposer *composer;
-	MailFolderView *mfv = (MailFolderView *)data;
-	MailComposerView *mcv;
 
-	mfv->priv->cancel = NULL;
-	composer = (EMsgComposer *)em_utils_edit_message (msg, folder);
-	mcv = (MailComposerView *)mail_shell_create_composer ();
-	mail_composer_view_replace_composer ((struct _MailComposerView *)mcv, (GtkWidget *)composer);
-}
-
-static void
-mfv_open_composer (MailFolderView *mfv, char *uid)
-{
-	e_msg_composer_set_lite();
-	mfv->priv->cancel = mail_get_messagex (mfv->priv->folder, uid, mfv_message_got, mfv, mail_msg_fast_ordered_push);
-}
 
 static EPopupItem mfv_popup_items[] = {
 //	{ E_POPUP_ITEM, "10.emfv.00", N_("_Reply to Sender"), emfv_popup_reply_sender, NULL, "mail-reply-sender", EM_POPUP_SELECT_ONE },
@@ -1150,15 +1184,15 @@ mfv_popup_items_free(EPopup *ep, GSList *items, void *data)
 {
 	g_slist_free(items);
 }
-
+#endif
 static void
 mfv_popup (MailFolderView *mfv, GdkEvent *event)
 {
-	GSList *menus = NULL;
-	GtkMenu *menu;
-	EMPopup *emp;
-	EMPopupTargetSelect *t;
-	int i;
+//	GSList *menus = NULL;
+//	GtkMenu *menu;
+//	EMPopup *emp;
+//	EMPopupTargetSelect *t;
+//	int i;
 	GtkTreePath *path = NULL;
 
 	if (gtk_tree_view_get_path_at_pos (mfv->tree, event->button.x, event->button.y, &path, NULL, NULL, NULL)) {
@@ -1176,7 +1210,8 @@ mfv_popup (MailFolderView *mfv, GdkEvent *event)
 	 *
 	 * This is the context menu shown on the message list.
 	 */
-	emp = em_popup_new("org.gnome.evolution.anjal.mail.folderview.popup");
+#warning "show-popup"	
+/*	emp = em_popup_new("org.gnome.evolution.anjal.mail.folderview.popup");
 	t = em_popup_target_new_select(emp, mfv->priv->folder, mfv->uri, mfv_get_selected_threads(mfv));
 	t->target.widget = (GtkWidget *)mfv;
 
@@ -1187,7 +1222,7 @@ mfv_popup (MailFolderView *mfv, GdkEvent *event)
 	menu = e_popup_create_menu_once((EPopup *)emp, (EPopupTarget *)t, 0);
 
 	gtk_menu_popup(menu, NULL, NULL, NULL, NULL, event->button.button, event->button.time);
-	
+*/	
 }
 
 static gboolean
@@ -1198,7 +1233,6 @@ mfv_button_clicked (GtkWidget *w, GdkEventButton *event, MailFolderView *mfv)
 	GtkTreePath *path = NULL;
 	GtkTreeViewColumn *column = NULL;
 
-
 	x = (int) event->x;
 	y = (int) event->y;
 
@@ -1253,13 +1287,18 @@ mfv_button_clicked (GtkWidget *w, GdkEventButton *event, MailFolderView *mfv)
 	return FALSE;
 }
 
+static void
+mfv_slider_pressed (GtkWidget *button, MailFolderView *shell)
+{
+	mail_folder_view_show_list (shell);
+}
+
 static void
 mfv_message_activated (GtkTreeView       *tree_view,  GtkTreePath       *path, GtkTreeViewColumn *column,  MailFolderView *shell)
 {
 	GtkTreeIter iter;
 	CamelMessageInfo *info = NULL;
 	gboolean outgoing = FALSE;
-
 	
 	if (em_utils_folder_is_drafts(shell->priv->folder, shell->uri)
 		|| em_utils_folder_is_outbox(shell->priv->folder, shell->uri))
@@ -1270,6 +1309,7 @@ mfv_message_activated (GtkTreeView       *tree_view,  GtkTreePath       *path, G
 		gtk_widget_hide ((GtkWidget *)shell->tree);
 		gtk_widget_hide (shell->priv->view_scroller);
 		gtk_widget_show ((GtkWidget *)shell->mail);
+		gtk_widget_show (shell->slidebox);
 		/* mail_float_bar_new (gtk_widget_get_ancestor (tree_view, GTK_TYPE_WINDOW)); */
 	}
 
@@ -1304,8 +1344,9 @@ mfv_tree_resize (GtkWidget *w, GtkAllocation *req, GtkTreeViewColumn *col)
 
 	gtk_tree_view_column_set_fixed_width (shell->priv->cols[2], 24);
 	/* gtk_tree_view_column_set_sizing (shell->priv->cols[2], GTK_TREE_VIEW_COLUMN_AUTOSIZE); */
+//	if ((req->width - (gtk_tree_view_column_get_fixed_width(col)+56) > 1) && GTK_WIDGET_VISIBLE(shell->priv->folder_pane)) 
 
-	if ((req->width - (gtk_tree_view_column_get_fixed_width(col)+56) > 1) && GTK_WIDGET_VISIBLE(shell->priv->folder_pane)) 
+	if ((req->width - (gtk_tree_view_column_get_fixed_width(col)+56) > 1)) 
 		gtk_tree_view_column_set_fixed_width (shell->priv->cols[1], req->width - 80);
 	gtk_tree_view_column_set_fixed_width (shell->priv->cols[2], 16);
 
@@ -1444,7 +1485,8 @@ static gboolean
 mfv_key_pressed (GtkWidget *w, GdkEventKey *event, MailFolderView *mfv)
 {
 	 gboolean shift = event->state & GDK_SHIFT_MASK ? TRUE : FALSE;
-	 
+	 gboolean ctrl = event->state & GDK_CONTROL_MASK ? TRUE : FALSE;
+
 	 switch (event->keyval) {
 	 case GDK_D:
 	 case GDK_d:		  
@@ -1461,6 +1503,12 @@ mfv_key_pressed (GtkWidget *w, GdkEventKey *event, MailFolderView *mfv)
 	 case GDK_m:
 		  mfv_read(mfv, !shift);
 		  break;
+	case GDK_w:
+	case GDK_W:
+		if (!ctrl)
+			return FALSE;
+		g_signal_emit (mfv, signals[VIEW_CLOSE], 0);			
+		break;
 	 default:
 		  return FALSE;
 	 }
@@ -1540,12 +1588,12 @@ void
 mail_folder_view_construct (MailFolderView *shell)
 {
 	MailFolderViewPrivate *priv = shell->priv;
-	GtkWidget *viewbox, *infolabel, *align;
+	GtkWidget *viewbox, *slidebox, *infolabel, *align, *tmp;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
 	GtkCellRenderer *hbox, *vbox;
 	GtkTreeSelection *sel;
-	
+
 	viewbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (viewbox);
 	infolabel = gtk_label_new (_("Please select a folder on the folder tree"));
@@ -1567,7 +1615,6 @@ mail_folder_view_construct (MailFolderView *shell)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->view_scroller),
 									GTK_POLICY_NEVER,
 									GTK_POLICY_AUTOMATIC);
-
 	gtk_box_pack_start ((GtkBox *)shell, viewbox, TRUE, TRUE, 0);
 	gtk_widget_show ((GtkWidget *)shell);
 	
@@ -1633,7 +1680,7 @@ mail_folder_view_construct (MailFolderView *shell)
 	else 
 		g_object_set_data ((GObject *)cell, "data", GINT_TO_POINTER (EMTS_COL_UNREAD));
 	custom_cell_renderer_vbox_append (vbox, cell);
-	shell->priv->unread_toggle = cell; /* Store the cell_render */
+	shell->priv->unread_toggle = g_object_ref_sink (cell); /* Store the cell_render */
 	g_object_set_data ((GObject *)cell, "mfv", shell);
 
 	/* dummy renderers for nice packing */
@@ -1676,7 +1723,7 @@ mail_folder_view_construct (MailFolderView *shell)
 		g_object_set_data ((GObject *)cell, "data", GINT_TO_POINTER (COL_FROM));
 	else 
 		g_object_set_data ((GObject *)cell, "data", GINT_TO_POINTER (EMTS_COL_FROM));
-	shell->priv->from_to = cell;
+	shell->priv->from_to = g_object_ref_sink (cell);
 	g_object_set_data ((GObject *)cell, "p-ellipsize", GINT_TO_POINTER (1));
 	g_object_set_data ((GObject *)cell, "p-markup", GINT_TO_POINTER (1));
 	g_object_set_data ((GObject *)cell, "p-bold", GINT_TO_POINTER (0));
@@ -1812,9 +1859,21 @@ mail_folder_view_construct (MailFolderView *shell)
 
 	gtk_tree_view_columns_autosize (shell->tree);
 
+	slidebox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start ((GtkBox *)viewbox, (GtkWidget *)slidebox, TRUE, TRUE, 0);
+	shell->slidebox = slidebox;
+
+	shell->slider = gtk_button_new ();
+	tmp = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+	gtk_container_add ((GtkContainer *)shell->slider, tmp);
+	gtk_widget_show_all (shell->slider);
+	gtk_box_pack_start ((GtkBox *)slidebox, (GtkWidget *)shell->slider, FALSE, FALSE, 2);
+	g_signal_connect (shell->slider, "clicked", G_CALLBACK(mfv_slider_pressed), shell);
+
 	shell->mail = mail_conv_view_new ();
-	gtk_box_pack_start ((GtkBox *)viewbox, (GtkWidget *)shell->mail, TRUE, TRUE, 0);
+	gtk_box_pack_start ((GtkBox *)slidebox, (GtkWidget *)shell->mail, TRUE, TRUE, 0);
 	gtk_widget_hide ((GtkWidget *)shell->mail);
+	gtk_widget_hide (shell->slidebox);
 }
 
 MailFolderView *
@@ -1914,16 +1973,14 @@ mail_folder_view_show_sort_popup  (MailFolderView *mfv, GtkWidget *button)
 void
 mail_folder_view_activate (MailFolderView *mfv, GtkWidget *tree, GtkWidget *folder_tree, GtkWidget *check_mail, GtkWidget *sort_by, GtkWidget *slider, gboolean act)
 {
-	 if (!folder_tree || !check_mail || !sort_by)
+	 if (!check_mail || !sort_by)
 		  return;
 	 if (mfv->priv->message_shown) {
-		  gtk_widget_show (folder_tree);
 		  g_signal_emit (mfv, signals[MESSAGE_SHOWN], 0);
 	 } else {
-		  gtk_widget_show(folder_tree);
-		  gtk_widget_hide (slider);
+		  ; //gtk_widget_hide (slider);
 	 }
-	 gtk_widget_set_sensitive (folder_tree, act);
+	 /* gtk_widget_set_sensitive (folder_tree, act);*/
 	 gtk_widget_set_sensitive (check_mail, act);
 	 gtk_widget_set_sensitive (sort_by, act);
 	 if (mfv->priv->folder) {
@@ -1957,6 +2014,10 @@ mail_folder_view_search_done (MailFolderView *mfv, CamelFolder *folder, const ch
 void
 mail_folder_view_set_search (MailFolderView *mfv, const char *search, const char *search_str)
 {
+	if ((search_str && mfv->search_str && strcmp (search_str, mfv->search_str) == 0) || 
+		((mfv->search_str == NULL|| *(mfv->search_str) == '\0') && (search_str == NULL|| *search_str == '\0')))
+		return;
+
 	if (mfv->search) {
 		g_free (mfv->search);
 		mfv->search = NULL;
@@ -1967,7 +2028,8 @@ mail_folder_view_set_search (MailFolderView *mfv, const char *search, const char
 		gboolean search_on = (search_str && *search_str) ? TRUE : FALSE;
 		char *str = g_strdup_printf ("%s %c%s%c", search_on ? _("Searching for ") : _("Clearing search"), search_on ? '\'' : ' ', search_on ? search_str : "", search_on ? '\'' : ' ');
 		mfv->search = search ? g_strdup(search) : NULL;
-
+		g_free (mfv->search_str);
+		mfv->search_str = g_strdup (search_str);
 		gtk_tree_view_set_model (GTK_TREE_VIEW (mfv->tree), NULL);
 		gtk_widget_show (mfv->priv->info_spinner);
 		gtk_widget_hide ((GtkWidget *)mfv->mail);
